package com.chinaitop.depot.grainAnalysis.controller;

import java.text.ParseException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.http.MediaType;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.grainAnalysis.model.GrainAnalysis;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisAbnormal;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisExample;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisMonthdata;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisTestdata;
import com.chinaitop.depot.grainAnalysis.model.TTestdata;
import com.chinaitop.depot.grainAnalysis.service.GrainAnalysisAbnormalService;
import com.chinaitop.depot.grainAnalysis.service.GrainAnalysisService;
import com.chinaitop.depot.grainAnalysis.service.TTestdataService;
import com.chinaitop.depot.unissoft.model.ResponseEntity;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping(value="/GrainAnalysis")
@Api(value= "GrainAnalysisController", description = "粮情分析周/月/季")
public class GrainAnalysisController {

	@Resource
	private GrainAnalysisService grainAnalysisService;
	@Resource
	private GrainAnalysisAbnormalService grainAnalysisAbnormalService;
	@Resource
	private TTestdataService tTestdataService;

	@RequestMapping(value="/pageList", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="周/月/季报列表", notes = "支持分页")
    @ApiImplicitParams({
    	@ApiImplicitParam(name="pageNum", value="页数", paramType="query"),
    	@ApiImplicitParam(name="pageSize", value="每页显示条数", paramType="query"),
    	@ApiImplicitParam(name="type", value="类型 1:周报 2月报 3季报", paramType="query"),
    	@ApiImplicitParam(name="houseId", value="仓房id", paramType="query"),
    	@ApiImplicitParam(name="jcqsrq", value="起始时间", paramType="query"),
    	@ApiImplicitParam(name="jczzrq", value="结束时间", paramType="query"),
		@ApiImplicitParam(name="jcnf", value="检查年份", paramType="query"),
		@ApiImplicitParam(name="jcyf", value="检查月份", paramType="query"),
		@ApiImplicitParam(name="jcjd", value="检查季度", paramType="query"),
		@ApiImplicitParam(name="orgId", value="组织机构Id", paramType="query")
    })
	public PageInfo<GrainAnalysis> pageList(HttpServletRequest request, Integer pageNum, Integer pageSize,
			Integer type, Integer houseId, @DateTimeFormat(pattern="yyyy-MM-dd")Date jcqsrq, @DateTimeFormat(pattern="yyyy-MM-dd")Date jczzrq,Integer jcnf, Integer jcyf, Integer jcjd,Integer orgId) {
		GrainAnalysisExample example = new GrainAnalysisExample();
		GrainAnalysisExample.Criteria criteria = example.createCriteria();
		if(orgId!=null){
			criteria.andOrgIdEqualTo(orgId);
		}
		if(type!=null){
			criteria.andTypeEqualTo(type);
		}
		if(houseId!=null){
			criteria.andHouseIdEqualTo(houseId);
		}
		if(jcqsrq!=null){
			criteria.andJcqsrqGreaterThanOrEqualTo(jcqsrq);
		}
		if(jczzrq!=null){
			criteria.andJczzrqLessThanOrEqualTo(jcqsrq);
		}
		if(jcnf!=null){
			criteria.andJcnfEqualTo(jcnf);
		}
		if(jcyf!=null){
			criteria.andJcyfEqualTo(jcyf);
		}
		if(jcjd!=null){
			criteria.andJcjdEqualTo(jcjd);
		}

		//只查旧版本的数据
		criteria.andZidIsNull();

		example.setOrderByClause("create_time desc");
		// 分页
		if (pageNum != null && pageSize != null) {
			PageHelper.startPage(pageNum, pageSize);
		}
		List<GrainAnalysis> list = grainAnalysisService.getList(example);
		PageInfo<GrainAnalysis> pageInfo = new PageInfo<>(list);
		return pageInfo;
	}

	@RequestMapping(value = "/addWeek", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="添加/编辑周报", notes = "添加周报")
    @ApiImplicitParams({
			@ApiImplicitParam(name = "grainAnalysisJson", value = "周报信息", paramType = "form")
    })
	public Map<String, Object> addWeek(String grainAnalysisJson) {
		Map<String, Object> modelMap = new HashMap<>();
		try {
			grainAnalysisService.saveOrUpdateWeek(grainAnalysisJson);
			modelMap.put("status", "success");
			modelMap.put("msg", "操作成功");
		} catch (Exception e) {
			modelMap.put("status", "error");
			modelMap.put("msg", e.getMessage());
			e.printStackTrace();
		}
		return modelMap;
	}

	@RequestMapping(value = "/addMonth", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="添加/编辑月报", notes = "添加月报")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "grainAnalysisJson", value = "月报信息", paramType = "form")
	})
	public Map<String, Object> addMonth(String grainAnalysisJson) {
		Map<String, Object> modelMap = new HashMap<>();
		// JSON字符串转对象
		ObjectMapper mapper = new ObjectMapper();
		try {
			GrainAnalysis grainAnalysis = mapper.readValue(grainAnalysisJson, GrainAnalysis.class);
			//异常信息与四无检查
			List<GrainAnalysisAbnormal> grainAnalysisAbnormalList = grainAnalysis.getGrainAnalysisAbnormalList();
			if(grainAnalysis.getId()!=null){
				grainAnalysisService.update(grainAnalysis);
				if(!CollectionUtils.isEmpty(grainAnalysisAbnormalList)){
					for (GrainAnalysisAbnormal grainAnalysisAbnormal : grainAnalysisAbnormalList) {
						grainAnalysisAbnormal.setGrainAnalysisId(grainAnalysis.getId());
						if(grainAnalysisAbnormal.getId()!=null){
							grainAnalysisAbnormalService.update(grainAnalysisAbnormal);
						}else{
							grainAnalysisAbnormalService.save(grainAnalysisAbnormal);
						}
					}
				}
			}else{
				grainAnalysis.setType(2);
				grainAnalysis.setCreateTime(new Date());
				grainAnalysisService.save(grainAnalysis);
				if(!CollectionUtils.isEmpty(grainAnalysisAbnormalList)){
					for (GrainAnalysisAbnormal grainAnalysisAbnormal : grainAnalysisAbnormalList) {
						grainAnalysisAbnormal.setGrainAnalysisId(grainAnalysis.getId());
						grainAnalysisAbnormalService.save(grainAnalysisAbnormal);
					}
				}
			}
			modelMap.put("status", "success");
		} catch (Exception e) {
			modelMap.put("status", "error");
			e.printStackTrace();
		}
		return modelMap;
	}

	@RequestMapping(value = "/addQuarter", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="添加/编辑季报", notes = "添加季报")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "grainAnalysisJson", value = "季报信息", paramType = "form")
	})
	public Map<String, Object> addQuarter(String grainAnalysisJson) {
		Map<String, Object> modelMap = new HashMap<>();
		// JSON字符串转对象
		ObjectMapper mapper = new ObjectMapper();
		try {
			GrainAnalysis grainAnalysis = mapper.readValue(grainAnalysisJson, GrainAnalysis.class);
			//异常信息与四无检查
			List<GrainAnalysisAbnormal> grainAnalysisAbnormalList = grainAnalysis.getGrainAnalysisAbnormalList();
			if(grainAnalysis.getId()!=null){
				grainAnalysisService.update(grainAnalysis);
				if(!CollectionUtils.isEmpty(grainAnalysisAbnormalList)){
					for (GrainAnalysisAbnormal grainAnalysisAbnormal : grainAnalysisAbnormalList) {
						grainAnalysisAbnormal.setGrainAnalysisId(grainAnalysis.getId());
						if(grainAnalysisAbnormal.getId()!=null){
							grainAnalysisAbnormalService.update(grainAnalysisAbnormal);
						}else{
							grainAnalysisAbnormalService.save(grainAnalysisAbnormal);
						}
					}
				}
			}else{
				grainAnalysis.setType(3);
				grainAnalysis.setCreateTime(new Date());
				grainAnalysisService.save(grainAnalysis);
				if(!CollectionUtils.isEmpty(grainAnalysisAbnormalList)){
					for (GrainAnalysisAbnormal grainAnalysisAbnormal : grainAnalysisAbnormalList) {
						grainAnalysisAbnormal.setGrainAnalysisId(grainAnalysis.getId());
						grainAnalysisAbnormalService.save(grainAnalysisAbnormal);
					}
				}
			}
			modelMap.put("status", "success");
		} catch (Exception e) {
			modelMap.put("status", "error");
			e.printStackTrace();
		}
		return modelMap;
	}

	@RequestMapping(value="/temperatureData", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="获取温湿度情况列表", notes = "获取温湿度情况列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name="orgId", value="组织机构Id", paramType="query"),
			@ApiImplicitParam(name="jcqsrq", value="起始时间", paramType="query"),
			@ApiImplicitParam(name="jczzrq", value="结束时间", paramType="query"),
			@ApiImplicitParam(name="jcnf", value="检查年份", paramType="query"),
			@ApiImplicitParam(name="jcyf", value="检查月份", paramType="query"),
			@ApiImplicitParam(name="jcjd", value="检查季度", paramType="query"),
			@ApiImplicitParam(name="houseId", value="仓房id", paramType="query")
	})
	public List<TTestdata> temperatureData(HttpServletRequest request, Integer orgId, @DateTimeFormat(pattern="yyyy-MM-dd")Date jcqsrq,
										   @DateTimeFormat(pattern="yyyy-MM-dd")Date jczzrq, Integer jcnf,
										   Integer jcyf, Integer jcjd, Integer houseId) {
		List<TTestdata> list = null;
		try {
			list = tTestdataService.getList(orgId,jcqsrq,jczzrq,jcnf,jcyf,jcjd,houseId);
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return list;
	}

	@RequestMapping(value="/info", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="获取详情", notes = "获取详情")
	@ApiImplicitParams({
			@ApiImplicitParam(name="id", value="主键id", paramType="query")
	})
	public GrainAnalysis info(String id) throws ParseException {
		
		GrainAnalysis grainAnalysis = null;
		try {
			grainAnalysis = grainAnalysisService.info(id);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return grainAnalysis;
	}

	@RequestMapping(value="/getWeekAnalysis", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="获取指定日期内有几个周报", notes = "获取指定日期内有几个周报")
	@ApiImplicitParams({
			@ApiImplicitParam(name="houseId", value="仓房id", paramType="query"),
			@ApiImplicitParam(name="jcnf", value="检查年份", paramType="query"),
			@ApiImplicitParam(name="jcyf", value="检查月份", paramType="query"),
			@ApiImplicitParam(name="orgId", value="组织机构Id", paramType="query")
	})
	public List<GrainAnalysis> getWeekAnalysis(HttpServletRequest request, Integer houseId, Integer jcnf, Integer jcyf,
										 Integer orgId) {
		List<GrainAnalysis> grainAnalysis = null;
		try {
			grainAnalysis = grainAnalysisService.getWeekAnalysis(houseId, jcnf, jcyf, orgId);
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return grainAnalysis;
	}

	@RequestMapping(value="/getMonthAnalysis", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="获取指定日期内上报了几个月报", notes = "获取指定日期内上报了几个月报")
	@ApiImplicitParams({
			@ApiImplicitParam(name="houseId", value="仓房id", paramType="query"),
			@ApiImplicitParam(name="jcnf", value="检查年份", paramType="query"),
			@ApiImplicitParam(name="jcjd", value="检查季度", paramType="query"),
			@ApiImplicitParam(name="orgId", value="组织机构Id", paramType="query")
	})
	public List<GrainAnalysis> getMonthAnalysis(HttpServletRequest request, Integer houseId, Integer jcnf, Integer jcjd,
											   Integer orgId) {
		List<GrainAnalysis> grainAnalysis = grainAnalysisService.getMonthAnalysis(houseId, jcnf, jcjd, orgId);
		return grainAnalysis;
	}

	@SuppressWarnings("all")
	@RequestMapping(value = "/removeGrainAnalisys", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
	@ApiOperation(value="删除一条仓房粮情分析数据", notes = "")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "id", value = "主键ID", paramType = "form"),
			@ApiImplicitParam(name = "sclx", value = "删除类型(0代表全删整条)", paramType = "form")
	})
	public ResponseEntity removeGrainAnalisys(String id, String sclx) {
		try {
			grainAnalysisService.removeGrainAnalisys(id, sclx);
		} catch (Exception e) {
			//e.printStackTrace();
			return ResponseEntity.failed(e.getMessage());
		}
		return ResponseEntity.ok();
	}

	@RequestMapping(value="/queryWeekAnalysis", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="选择周报", notes = "")
	@ApiImplicitParams({
			@ApiImplicitParam(name="ssyf", value="所属月份", paramType="query"),
			@ApiImplicitParam(name="houseId", value="仓房ID", paramType="query")
	})
	public List<GrainAnalysisMonthdata> queryWeekAnalysis(String ssyf, Integer houseId) {
		List<GrainAnalysisMonthdata> list = null;
		try {
			list = grainAnalysisService.queryWeekAnalysis(ssyf, houseId);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return list;
	}

	@RequestMapping(value="/queryGrainAnalysisTestdata", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="查看仓房温湿度情况", notes = "")
	@ApiImplicitParams({
			@ApiImplicitParam(name="ids", value="仓房所属周报的id集合", paramType="query"),
			@ApiImplicitParam(name="gn_type", value="功能类型(month是半月报, quarter代表季报)", paramType="query"),
			@ApiImplicitParam(name="sbzq", value="上报周期", paramType="query"),
			@ApiImplicitParam(name="houseId", value="仓房ID", paramType="query"),
	})
	public List<GrainAnalysisTestdata> queryGrainAnalysisTestdata(String gn_type, String ids, String sbzq, Integer houseId) {
		List<GrainAnalysisTestdata> list = null;
		try {
			list = grainAnalysisService.queryGrainAnalysisTestdata(ids, gn_type, sbzq, houseId);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return list;
	}
}
