package com.chinaitop.depot.grainAnalysis.controller;

import java.text.ParseException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.grainAnalysis.model.GrainAnalysis;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisTestdata;
import com.chinaitop.depot.grainAnalysis.model.StorageGrainAnalysis;
import com.chinaitop.depot.grainAnalysis.model.TTestdata;
import com.chinaitop.depot.grainAnalysis.service.StorageGrainAnalysisService;
import com.chinaitop.depot.unissoft.model.ResponseEntity;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

@SuppressWarnings("all")
@RestController
@RequestMapping(value="/storageGrainAnalysis")
@Api(value= "StorageGrainAnalysisController", description = "新的粮情分析周/月/季")
public class StorageGrainAnalysisController {

	@Resource
	private StorageGrainAnalysisService storageGrainAnalysisService;

	@RequestMapping(value = "/getList", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "查询报表列表", notes = "支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页数", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "提交日期开始时间", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "提交日期结束时间", paramType = "query"),
            @ApiImplicitParam(name = "orgId", value = "库ID", paramType = "query"),
            @ApiImplicitParam(name = "dataType", value = "数据类型(0:周报,1:半月报,2:季报)", paramType = "query"),
            @ApiImplicitParam(name = "ssyf", value = "所属月份", paramType = "query"),
            @ApiImplicitParam(name = "dataStatus", value="数据状态(0:保存,1:提交)", paramType = "query")
    })
	public ResponseEntity getList(Integer pageNum, Integer pageSize, String startTime, String endTime, 
			Integer orgId, String dataType, String ssyf, String dataStatus) {
		List<StorageGrainAnalysis> list = null;
        PageInfo<StorageGrainAnalysis> pageInfo = null;
        try {
        	if (null != pageNum && null != pageSize) {
        		PageHelper.startPage(pageNum, pageSize);
        	}
			list = storageGrainAnalysisService.getList(startTime, endTime, orgId, dataType, ssyf, dataStatus);
			pageInfo = new PageInfo<StorageGrainAnalysis>(list);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return ResponseEntity.ok(pageInfo);
	}

	@RequestMapping(value = "/selectClcf", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "查询当前粮库储粮的仓房(不含油罐)", notes = "")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orgId", value = "库ID", paramType = "query")
    })
	public ResponseEntity selectClcf(String orgId) {
		List<Map<String,Object>> list = null;
        try {
			list = storageGrainAnalysisService.selectClcf(orgId);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return ResponseEntity.ok(list);
	}
	
	@RequestMapping(value = "/save", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "生成报表", notes = "")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "grainJson", value = "主表数据对象", paramType = "form"),
            @ApiImplicitParam(name = "houseGrainJson", value = "子表数据对象", paramType = "form")
    })
	public Map<String, String> save(String grainJson, String houseGrainJson) {
		Map<String, String> map = new HashMap<>();
        try {
        	map = storageGrainAnalysisService.save(grainJson, houseGrainJson);
		} catch (Exception e) {
			map.put("status", "500");
			map.put("msg", e.getMessage());
			e.printStackTrace();
		}
		return map;
	}

	@RequestMapping(value = "/findByGrainAnalysis", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "查询一个报表下面的仓房明细数据", notes = "")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主表数据ID", paramType = "query")
    })
	public Map<String, Object> findByGrainAnalysis(String id) {
		Map<String, Object> map = new HashMap<>();
		try {
			List<GrainAnalysis> list_spt = storageGrainAnalysisService.selectGrainAnalysisSpt(id);
			StorageGrainAnalysis storageGrainAnalysis = storageGrainAnalysisService.findByIdStorageGrainAnalysis(id);
			map.put("storageGrainAnalysis", storageGrainAnalysis);
			map.put("grainAnalysisList", list_spt);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return map;
	}
	
	@RequestMapping(value="/temperatureDataNew", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="获取温湿度情况列表(新版)", notes = "")
	@ApiImplicitParams({
			@ApiImplicitParam(name="jcqsrq", value="起始时间", paramType="query"),
			@ApiImplicitParam(name="jczzrq", value="结束时间", paramType="query"),
			@ApiImplicitParam(name="houseId", value="仓房id", paramType="query")
	})
	public List<GrainAnalysisTestdata> temperatureDataNew(String jcqsrq, String jczzrq, Integer houseId) {
		List<GrainAnalysisTestdata> list = null;
		try {
			list = storageGrainAnalysisService.getTemperatureDataNewList(jcqsrq, jczzrq, houseId);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return list;
	}

	@RequestMapping(value = "/submitStorageGrainAnalysis", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
    @ApiOperation(value = "提交整条报表", notes = "")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "报表数据id", paramType = "form")
    })
	public Map<String, String> submitStorageGrainAnalysis(String id) {
		Map<String, String> map = new HashMap<>();
        try {
        	map = storageGrainAnalysisService.submitStorageGrainAnalysis(id);
		} catch (Exception e) {
			map.put("status", "500");
			map.put("msg", e.getMessage());
			e.printStackTrace();
		}
		return map;
	}

	@RequestMapping(value = "/getListSpt", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value = "查询报表列表(市平台专用)", notes = "支持分页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "页数", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
            @ApiImplicitParam(name = "startTime", value = "提交日期开始时间", paramType = "query"),
            @ApiImplicitParam(name = "endTime", value = "提交日期结束时间", paramType = "query"),
            @ApiImplicitParam(name = "kqbm", value = "库区编码(国标编码)", paramType = "query"),
            @ApiImplicitParam(name = "dataType", value = "数据类型(0:周报,1:半月报,2:季报)", paramType = "query"),
            @ApiImplicitParam(name = "ssyf", value = "所属年份", paramType = "query")
    })
	public ResponseEntity getListSpt(Integer pageNum, Integer pageSize, String startTime, String endTime, 
			String kqbm, String dataType, String ssyf) {
		List<StorageGrainAnalysis> list = null;
        PageInfo<StorageGrainAnalysis> pageInfo = null;
        try {
        	if (null != pageNum && null != pageSize) {
        		PageHelper.startPage(pageNum, pageSize);
        	}
			list = storageGrainAnalysisService.getListSpt(startTime, endTime, kqbm, dataType, ssyf);
			pageInfo = new PageInfo<StorageGrainAnalysis>(list);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return ResponseEntity.ok(pageInfo);
	}

}
