package com.chinaitop.depot.grainAnalysis.service;

import java.util.List;
import java.util.Map;

import com.chinaitop.depot.grainAnalysis.model.GrainAnalysis;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisTestdata;
import com.chinaitop.depot.grainAnalysis.model.StorageGrainAnalysis;

public interface StorageGrainAnalysisService {

	/**
	 * 查询周报列表
	 * @param pageNum
	 * @param pageSize
	 * @param startTime
	 * @param endTime
	 * @param orgId
	 * @param dataType 数据类型(0:周报,1:半月报,2:季报)
	 * @param ssyf 所属月份
	 * @param 数据状态(0:保存,1:提交)
	 * @return
	 * @throws Exception
	 */
	List<StorageGrainAnalysis> getList(String startTime, String endTime, Integer orgId, String dataType, String ssyf, String dataStatus) throws Exception;

	/**
	 * 查询一个库区中有粮食的仓房(不含油罐)
	 * @param orgId
	 * @return
	 * @throws Exception
	 */
	List<Map<String, Object>> selectClcf(String orgId) throws Exception;

	/**
	 * 查询一个报表下面的仓房明细数据
	 * @param id 主表ID
	 * @return
	 * @throws Exception
	 */
	List<GrainAnalysis> selectGrainAnalysis(String id) throws Exception;
	
	/**
	 * 查询一个报表下面的仓房明细数据(市平台查询用)
	 * @param zid 主表ID
	 * @return
	 * @throws Exception
	 */
	List<GrainAnalysis> selectGrainAnalysisSpt(String zid) throws Exception;

	/**
     * 修改一条主表数据
     * @param id 主表ID
     * @throws Exception
     */
	void updateStorageGrainAnalysis(String id) throws Exception;

    /**
     * 修改一条仓房明细表数据
     * @param id 仓房明细表ID
     * @throws Exception
     */
	void updateGrainAnalysis(String id) throws Exception;

	/**
     * 删除一条主表数据
     * @param id 主表ID
     * @throws Exception
     */
	void removeStorageGrainAnalysis(String id) throws Exception;

	/**
     * 删除一条仓房明细表数据
     * @param id 仓房明细表ID
     * @throws Exception
     */
	void removeGrainAnalysis(String id) throws Exception;

	/**
	 * 生成报表数据
	 * @param grainJson 主表数据对象
	 * @param houseGrainJson 子表数据对象
	 * @return
	 */
	Map<String, String> save(String grainJson, String houseGrainJson) throws Exception;

	/**
	 * 按主键ID查询一条主表数据
	 * @param id
	 * @return
	 * @throws Exception
	 */
	StorageGrainAnalysis findByIdStorageGrainAnalysis(String id) throws Exception;

	/**
	 * 查询一个仓房在一个时间范围内的温度数据(只取每天的最后一条)
	 * @param jcqsrq 起始时间
	 * @param jczzrq 结束时间
	 * @param houseId 仓房ID
	 * @return
	 * @throws Exception
	 */
	List<GrainAnalysisTestdata> getTemperatureDataNewList(String jcqsrq, String jczzrq, Integer houseId) throws Exception;

	/**
	 * 提交整条报表
	 * @param id 报表数据主键ID
	 * @return
	 * @throws Exception
	 */
	Map<String, String> submitStorageGrainAnalysis(String id) throws Exception;

	/**
	 * 查询周报列表(市平台专用)
	 * @param pageNum
	 * @param pageSize
	 * @param startTime
	 * @param endTime
	 * @param kqbm
	 * @param dataType 数据类型(0:周报,1:半月报,2:季报)
	 * @param ssyf 所属月份
	 * @return
	 * @throws Exception
	 */
	List<StorageGrainAnalysis> getListSpt(String startTime, String endTime, String kqbm, String dataType, String ssyf) throws Exception;
}
