package com.chinaitop.depot.grainAnalysis.service.impl;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import com.chinaitop.depot.grainAnalysis.mapper.GrainAnalysisAbnormalMapper;
import com.chinaitop.depot.grainAnalysis.mapper.GrainAnalysisMapper;
import com.chinaitop.depot.grainAnalysis.mapper.GrainAnalysisMonthdataMapper;
import com.chinaitop.depot.grainAnalysis.mapper.GrainAnalysisTestdataMapper;
import com.chinaitop.depot.grainAnalysis.mapper.StorageGrainAnalysisMapper;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysis;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisAbnormal;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisAbnormalExample;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisExample;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisMonthdata;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisMonthdataExample;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisTestdata;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisTestdataExample;
import com.chinaitop.depot.grainAnalysis.model.StorageGrainAnalysis;
import com.chinaitop.depot.grainAnalysis.model.StorageGrainAnalysisExample;
import com.chinaitop.depot.grainAnalysis.model.TTestdata;
import com.chinaitop.depot.grainAnalysis.service.GrainAnalysisService;
import com.chinaitop.depot.grainAnalysis.service.StorageGrainAnalysisService;
import com.chinaitop.depot.grainAnalysis.service.TTestdataService;
import com.fasterxml.jackson.databind.ObjectMapper;

@Service
public class GrainAnalysisServiceImpl implements GrainAnalysisService {

	@Resource
	private StorageGrainAnalysisService storageGrainAnalysisService;
	@Resource
	private StorageGrainAnalysisMapper storageGrainAnalysisMapper;
	@Resource
	private GrainAnalysisMapper grainAnalysisMapper;
	@Resource
	private TTestdataService tTestdataService;
	@Resource
	private GrainAnalysisTestdataMapper grainAnalysisTestdataMapper;
	@Resource
	private GrainAnalysisAbnormalMapper grainAnalysisAbnormalMapper;
	@Resource
	private GrainAnalysisMonthdataMapper grainAnalysisMonthdataMapper;

	@Override
	public List<GrainAnalysis> getList(GrainAnalysisExample example) {
		return grainAnalysisMapper.selectByExample(example);
	}

	@Override
	public void save(GrainAnalysis grainAnalysisAbnormal) {
		grainAnalysisMapper.insert(grainAnalysisAbnormal);
	}

	@Override
	public void update(GrainAnalysis grainAnalysisAbnormal) {
		grainAnalysisMapper.updateByPrimaryKey(grainAnalysisAbnormal);
	}

	@Override
	public GrainAnalysis getById(String id) {
		return grainAnalysisMapper.selectByPrimaryKey(id);
	}

	@Override
	public List<GrainAnalysis> getMonthAnalysis(Integer houseId, Integer jcnf, Integer jcjd, Integer orgId) {
		GrainAnalysisExample example = new GrainAnalysisExample();
		GrainAnalysisExample.Criteria criteria = example.createCriteria();
		if(orgId!=null){
			criteria.andOrgIdEqualTo(orgId);
		}
		criteria.andTypeEqualTo(2);
		if(houseId!=null){
			criteria.andHouseIdEqualTo(houseId);
		}
		if(jcnf!=null){
			criteria.andJcnfEqualTo(jcnf);
		}
		if(jcjd !=null ){
			if(jcjd == 1){
				criteria.andJcyfGreaterThanOrEqualTo(1);
				criteria.andJcyfLessThanOrEqualTo(3);
			}else if(jcjd == 2){
				criteria.andJcyfGreaterThanOrEqualTo(4);
				criteria.andJcyfLessThanOrEqualTo(6);
			}else if(jcjd == 3){
				criteria.andJcyfGreaterThanOrEqualTo(7);
				criteria.andJcyfLessThanOrEqualTo(9);
			}else {
				criteria.andJcyfGreaterThanOrEqualTo(10);
				criteria.andJcyfLessThanOrEqualTo(12);
			}
		}
		example.setOrderByClause("jcyf desc");
		List<GrainAnalysis> list = grainAnalysisMapper.selectByExample(example);
		if(!CollectionUtils.isEmpty(list)){
			for (GrainAnalysis grainAnalysis : list) {
				List<TTestdata> tTestdata = null;
				try {
					tTestdata = tTestdataService.getList(orgId,null,null,grainAnalysis.getJcnf(),grainAnalysis.getJcyf(),null,houseId);
				} catch (ParseException e) {
					e.printStackTrace();
				}
				if(!CollectionUtils.isEmpty(tTestdata)){
					TTestdata onlyData = tTestdata.get(0);
					List<TTestdata> onlyDatas = new ArrayList<>();
					onlyDatas.add(onlyData);
					grainAnalysis.settTestdata(onlyDatas);
				}
			}
		}
		return list;
	}

	@Override
	public List<GrainAnalysis> getWeekAnalysis(Integer houseId, Integer jcnf, Integer jcyf, Integer orgId) throws ParseException {
		GrainAnalysisExample example = new GrainAnalysisExample();
		GrainAnalysisExample.Criteria criteria = example.createCriteria();
		if(orgId!=null){
			criteria.andOrgIdEqualTo(orgId);
		}
		criteria.andTypeEqualTo(1);
		if(houseId!=null){
			criteria.andHouseIdEqualTo(houseId);
		}
		Date startDate = null;
		Date endDate = null;
		if(jcnf!=null && jcyf!=null){
			startDate = getFirstDayOfMonth(jcnf,jcyf);
			endDate = getLastDayOfMonth(jcnf,jcyf);
		}
		if(startDate!=null){
			criteria.andJcqsrqGreaterThanOrEqualTo(startDate);
		}
		if(endDate!=null){
			criteria.andJcqsrqLessThanOrEqualTo(endDate);
		}
		example.setOrderByClause("jcqsrq desc");
		List<GrainAnalysis> list = grainAnalysisMapper.selectByExample(example);
		if(!CollectionUtils.isEmpty(list)){
			for (GrainAnalysis grainAnalysis : list) {
				List<TTestdata> tTestdata = tTestdataService.getList(orgId,grainAnalysis.getJcqsrq(),grainAnalysis.getJczzrq(),null,null,null,houseId);
				if(!CollectionUtils.isEmpty(tTestdata)){
					TTestdata onlyData = tTestdata.get(0);
					List<TTestdata> onlyDatas = new ArrayList<>();
					onlyDatas.add(onlyData);
					grainAnalysis.settTestdata(onlyDatas);
				}
			}
		}
		return list;
	}

	//获取某年某月第一天
	public static Date getFirstDayOfMonth(Integer jcnf, Integer jcyf) throws ParseException {
		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.YEAR,jcnf);    //设置年份
		cal.set(Calendar.MONTH, jcyf-1);  //设置月份
		int firstDay = cal.getActualMinimum(Calendar.DAY_OF_MONTH); //获取某月最小天数
		cal.set(Calendar.DAY_OF_MONTH, firstDay);   //设置日历中月份的最小天数
		//格式化日期
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
		return sdf.parse(sdf.format(cal.getTime()));

	}

	//获取某年某月最后一天
	public static Date getLastDayOfMonth(Integer jcnf, Integer jcyf) throws ParseException {

		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.YEAR,jcnf);    //设置年份
		cal.set(Calendar.MONTH, jcyf-1);  //设置月份
		int lastDay = cal.getActualMaximum(Calendar.DAY_OF_MONTH);  //获取某月最大天数
		cal.set(Calendar.DAY_OF_MONTH, lastDay);    //设置日历中月份的最大天数
		//格式化日期
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd 23:59:59");
		return sdf.parse(sdf.format(cal.getTime()));
	}

	@Override
	public void saveOrUpdateWeek(String grainAnalysisJson) throws Exception {
		ObjectMapper mapper = new ObjectMapper();
		GrainAnalysis grainAnalysis = mapper.readValue(grainAnalysisJson, GrainAnalysis.class);
		//报表主表数据
		StorageGrainAnalysis storageGrainAnalysis = storageGrainAnalysisMapper.selectByPrimaryKey(grainAnalysis.getZid());
		//异常信息与四无检查
		List<GrainAnalysisAbnormal> grainAnalysisAbnormalList = grainAnalysis.getGrainAnalysisAbnormalList();
		//新版的温度数据
		List<GrainAnalysisTestdata> analysisTestdataList = grainAnalysis.getAnalysisTestdata();
		//月报下挂载的周报数据/季报下挂载的月报数据
		List<GrainAnalysisMonthdata> analysisMonthdataList = grainAnalysis.getAnalysisMonthdata();
		if(grainAnalysis.getId()!=null){
			grainAnalysisMapper.updateByPrimaryKey(grainAnalysis);
			if(!CollectionUtils.isEmpty(grainAnalysisAbnormalList)){
				for (GrainAnalysisAbnormal grainAnalysisAbnormal : grainAnalysisAbnormalList) {
					grainAnalysisAbnormal.setGrainAnalysisId(grainAnalysis.getId());
					if(grainAnalysisAbnormal.getId()!=null){
						grainAnalysisAbnormalMapper.updateByPrimaryKey(grainAnalysisAbnormal);
					}else{
						grainAnalysisAbnormalMapper.insert(grainAnalysisAbnormal);
					}
				}
			}
			if (!CollectionUtils.isEmpty(analysisTestdataList)) {
				//先删除
				GrainAnalysisTestdataExample grainAnalysisTestdataExample = new GrainAnalysisTestdataExample();
				GrainAnalysisTestdataExample.Criteria criteria = grainAnalysisTestdataExample.createCriteria();
				criteria.andGrainAnalysisIdEqualTo(grainAnalysis.getId());
				grainAnalysisTestdataMapper.deleteByExample(grainAnalysisTestdataExample);
				//在新增
				Date date = new Date();
				for (GrainAnalysisTestdata g : analysisTestdataList) {
					g.setId(UUID.randomUUID().toString().replace("-", ""));
					g.setGrainAnalysisId(grainAnalysis.getId());
					g.setOrgId(grainAnalysis.getOrgId());
					g.setCjsj(date);
					g.setUpdatetime(date);
					grainAnalysisTestdataMapper.insert(g);
				}
			}
			if (!CollectionUtils.isEmpty(analysisMonthdataList) && "2".equals(storageGrainAnalysis.getDataType())) {
				//先删除
				GrainAnalysisMonthdataExample grainAnalysisMonthdataExample = new GrainAnalysisMonthdataExample();
				GrainAnalysisMonthdataExample.Criteria grainAnalysisMonthdataCriteria = grainAnalysisMonthdataExample.createCriteria();
				grainAnalysisMonthdataCriteria.andGrainAnalysisIdEqualTo(grainAnalysis.getId());
				grainAnalysisMonthdataMapper.deleteByExample(grainAnalysisMonthdataExample);
				//在新增
				Date date = new Date();
				for (GrainAnalysisMonthdata m : analysisMonthdataList) {
					m.setId(UUID.randomUUID().toString().replace("-", ""));
					m.setGrainAnalysisId(grainAnalysis.getId());
					m.setOrgId(grainAnalysis.getOrgId());
					m.setCjsj(date);
					m.setUpdatetime(date);
					grainAnalysisMonthdataMapper.insert(m);
				}
			}
		}else{
			grainAnalysis.setType(1);
			grainAnalysis.setCreateTime(new Date());
			grainAnalysisMapper.insert(grainAnalysis);
			if(!CollectionUtils.isEmpty(grainAnalysisAbnormalList)){
				for (GrainAnalysisAbnormal grainAnalysisAbnormal : grainAnalysisAbnormalList) {
					grainAnalysisAbnormal.setGrainAnalysisId(grainAnalysis.getId());
					grainAnalysisAbnormalMapper.insert(grainAnalysisAbnormal);
				}
			}
		}
	}

	@Override
	public GrainAnalysis info(String id) throws Exception {
		/* 查询仓房粮情分析表数据 */
		GrainAnalysis grainAnalysis = grainAnalysisMapper.selectByPrimaryKey(id);

		/* 查询报表主表数据 */
		StorageGrainAnalysis storageGrainAnalysis = storageGrainAnalysisMapper.selectByPrimaryKey(grainAnalysis.getZid());
		if (null != storageGrainAnalysis) {
			grainAnalysis.setStorageGrainAnalysis(storageGrainAnalysis);
		}

		/* 查询子表数据 */
		GrainAnalysisAbnormalExample example = new GrainAnalysisAbnormalExample();
		GrainAnalysisAbnormalExample.Criteria criteria = example.createCriteria();
		criteria.andGrainAnalysisIdEqualTo(grainAnalysis.getId());
		List<GrainAnalysisAbnormal> grainAnalysisAbnormalList = grainAnalysisAbnormalMapper.selectByExample(example);

		/* 赋值给异常情况与四无检查 */
		if(!CollectionUtils.isEmpty(grainAnalysisAbnormalList)){
			grainAnalysis.setGrainAnalysisAbnormalList(grainAnalysisAbnormalList);
		}
		//查询旧版的温度数据
		List<TTestdata> list = tTestdataService.getList(grainAnalysis.getOrgId(),grainAnalysis.getJcqsrq(),grainAnalysis.getJczzrq(),
					grainAnalysis.getJcnf(),grainAnalysis.getJcyf(),grainAnalysis.getJcjd(),grainAnalysis.getHouseId());
		
		//查询新版的温度数据
		GrainAnalysisTestdataExample testdataExample = new GrainAnalysisTestdataExample();
		GrainAnalysisTestdataExample.Criteria testdataCriteria = testdataExample.createCriteria();
		testdataCriteria.andGrainAnalysisIdEqualTo(id);
		
		List<GrainAnalysisTestdata> grainAnalysisTestdataList = grainAnalysisTestdataMapper.selectByExample(testdataExample);
		if (!CollectionUtils.isEmpty(grainAnalysisTestdataList)) {
			grainAnalysis.setAnalysisTestdata(grainAnalysisTestdataList);
		}
		
		//查询月报挂载的周报数据/季报挂载的月报数据
		GrainAnalysisMonthdataExample monthdataExample = new GrainAnalysisMonthdataExample();
		GrainAnalysisMonthdataExample.Criteria monthdataCriteria = monthdataExample.createCriteria();
		monthdataCriteria.andGrainAnalysisIdEqualTo(id);
		
		List<GrainAnalysisMonthdata> grainAnalysisMonthdataList = grainAnalysisMonthdataMapper.selectByExample(monthdataExample);
		if (!CollectionUtils.isEmpty(grainAnalysisMonthdataList)) {
			grainAnalysis.setAnalysisMonthdata(grainAnalysisMonthdataList);
		}
		
		if (!CollectionUtils.isEmpty(list)) {
			if (grainAnalysis.getType() == 1) {//周报
				grainAnalysis.settTestdata(list);
			} else if (grainAnalysis.getType() == 2) {//月报
                //查询对应年份月份的周报的数据，然后取每条周报数据所在时间范围内的第一条测温数据
				List<TTestdata> onlyDatas = new ArrayList<>();
				List<GrainAnalysis> grainAnalysisList = getWeekAnalysis(grainAnalysis.getHouseId(), grainAnalysis.getJcnf(), grainAnalysis.getJcyf(), grainAnalysis.getOrgId());
				if(!CollectionUtils.isEmpty(grainAnalysisList) && !CollectionUtils.isEmpty(grainAnalysisAbnormalList)){
					for (GrainAnalysis analysis : grainAnalysisList) {
						if (!CollectionUtils.isEmpty(analysis.gettTestdata())) {
							onlyDatas.addAll(analysis.gettTestdata());
						}
					}
				}
				grainAnalysis.settTestdata(onlyDatas);
			}else{//以下是季报
				List<TTestdata> onlyDatas = new ArrayList<>();
				List<GrainAnalysis> grainAnalysisList = getMonthAnalysis(grainAnalysis.getHouseId(), grainAnalysis.getJcnf(), grainAnalysis.getJcjd(), grainAnalysis.getOrgId());
				if(!CollectionUtils.isEmpty(grainAnalysisList) && !CollectionUtils.isEmpty(grainAnalysisAbnormalList)){
					for (GrainAnalysis analysis : grainAnalysisList) {
						if (!CollectionUtils.isEmpty(analysis.gettTestdata())) {
							onlyDatas.addAll(analysis.gettTestdata());
						}
					}
				}
				grainAnalysis.settTestdata(onlyDatas);
			}

		}
		return grainAnalysis;
	}

	@Override
	public void removeGrainAnalisys(String id, String sclx) throws Exception {

		if ("0".equals(sclx)) {
			storageGrainAnalysisMapper.deleteByPrimaryKey(id);

			GrainAnalysisExample grainAnalysisExample = new GrainAnalysisExample();
			GrainAnalysisExample.Criteria grainAnalysisCriteria = grainAnalysisExample.createCriteria();
			grainAnalysisCriteria.andZidEqualTo(id);
			List<GrainAnalysis> list = grainAnalysisMapper.selectByExample(grainAnalysisExample);
			for (int i = 0; i < list.size(); i++) {
				String zid = list.get(i).getId();
				removeObj(zid);
			}
		} else {
			removeObj(id);
		}

	}

	public void removeObj(String id)  throws Exception {
		GrainAnalysisTestdataExample testdataExample = new GrainAnalysisTestdataExample();
		GrainAnalysisTestdataExample.Criteria testdataCriteria = testdataExample.createCriteria();
		testdataCriteria.andGrainAnalysisIdEqualTo(id);
		grainAnalysisTestdataMapper.deleteByExample(testdataExample);
		
		GrainAnalysisMonthdataExample monthdataExample = new GrainAnalysisMonthdataExample();
		GrainAnalysisMonthdataExample.Criteria monthdataCriteria = monthdataExample.createCriteria();
		monthdataCriteria.andGrainAnalysisIdEqualTo(id);
		grainAnalysisMonthdataMapper.deleteByExample(monthdataExample);
		
		grainAnalysisMapper.deleteByPrimaryKey(id);
	}

	@Override
	public List<GrainAnalysisMonthdata> queryWeekAnalysis(String ssyf, Integer houseId) throws Exception {
		
		//查询这个仓房所属月份到的周报信息
		Map<String, Object> param_map = new HashMap<>();
		param_map.put("houseId", houseId);
		param_map.put("ssyf", ssyf);

		List<GrainAnalysisMonthdata> list = grainAnalysisMonthdataMapper.queryWeekAnalysis(param_map);
		return list;
	}

	@Override
	public List<GrainAnalysisTestdata> queryGrainAnalysisTestdata(String ids, String gn_type, String sbzq, Integer houseId) throws Exception {
		List<GrainAnalysisTestdata> list = null;
		if ("month".equals(gn_type) && StringUtils.isNotBlank(ids)) {
			String []id = ids.split(",");
			List<String> grainAnlysisIdList = new ArrayList<>();
			for (int i = 0; i < id.length; i++) {
				grainAnlysisIdList.add(id[i]);
			}
			GrainAnalysisTestdataExample testdataExample = new GrainAnalysisTestdataExample();
			GrainAnalysisTestdataExample.Criteria testdataCriteria = testdataExample.createCriteria();
			testdataCriteria.andGrainAnalysisIdIn(grainAnlysisIdList);
			testdataExample.setOrderByClause("t_time ");
			list = grainAnalysisTestdataMapper.selectByExample(testdataExample);
		}
		if ("quarter".equals(gn_type) && StringUtils.isNotBlank(sbzq)) {
			//年份
			String jcnd = sbzq.substring(0, 4);
			//获取季度
			List<String> ssyf_list = new ArrayList<>();
			if (sbzq.indexOf("一") != -1) {//一季度
				ssyf_list.add(jcnd+"-01");
				ssyf_list.add(jcnd+"-02");
				ssyf_list.add(jcnd+"-03");
			} else if (sbzq.indexOf("二") != -1) {
				ssyf_list.add(jcnd+"-04");
				ssyf_list.add(jcnd+"-05");
				ssyf_list.add(jcnd+"-06");
			} else if (sbzq.indexOf("三") != -1) {
				ssyf_list.add(jcnd+"-07");
				ssyf_list.add(jcnd+"-08");
				ssyf_list.add(jcnd+"-09");
			} else if (sbzq.indexOf("四") != -1) {
				ssyf_list.add(jcnd+"-10");
				ssyf_list.add(jcnd+"-11");
				ssyf_list.add(jcnd+"-12");
			}
			//查到这个季度的所有周报
			StorageGrainAnalysisExample storageGrainAnalysisExample = new StorageGrainAnalysisExample();
			StorageGrainAnalysisExample.Criteria storageGrainAnalysisCriteria = storageGrainAnalysisExample.createCriteria();
			storageGrainAnalysisCriteria.andDataTypeEqualTo("1");//周报
			storageGrainAnalysisCriteria.andDataStatusEqualTo("1");//已提交的数据
			storageGrainAnalysisCriteria.andSsyfIn(ssyf_list);
			List<StorageGrainAnalysis> StorageGrainAnalysis_list = storageGrainAnalysisMapper.selectByExample(storageGrainAnalysisExample);
			
			//查到这个仓这些周报下的仓分析数据
			List<String> grainAnalysis_list = null;
			if (null != StorageGrainAnalysis_list && StorageGrainAnalysis_list.size() > 0) {
				grainAnalysis_list = StorageGrainAnalysis_list.stream().map(StorageGrainAnalysis::getId).collect(Collectors.toList());
			}
			GrainAnalysisExample grainAnalysisExample = new GrainAnalysisExample();
			GrainAnalysisExample.Criteria grainAnalysisCriteria = grainAnalysisExample.createCriteria();
			grainAnalysisCriteria.andZidIn(grainAnalysis_list);
			grainAnalysisCriteria.andHouseIdEqualTo(houseId);
			List<GrainAnalysis> grainAnalysisList = grainAnalysisMapper.selectByExample(grainAnalysisExample);
			if (null != grainAnalysisList && grainAnalysisList.size() > 0) {
				//把数据中的所有ID值取出来
				List<String> grainAnalysisId_list = grainAnalysisList.stream().map(GrainAnalysis::getId).collect(Collectors.toList());
				
				GrainAnalysisTestdataExample testdataExample = new GrainAnalysisTestdataExample();
				GrainAnalysisTestdataExample.Criteria testdataCriteria = testdataExample.createCriteria();
				testdataCriteria.andGrainAnalysisIdIn(grainAnalysisId_list);
				testdataExample.setOrderByClause("t_time ");
				list = grainAnalysisTestdataMapper.selectByExample(testdataExample);
			}
		}
		return list;
	}
}
