package com.chinaitop.depot.grainAnalysis.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.grainAnalysis.mapper.GrainAnalysisMapper;
import com.chinaitop.depot.grainAnalysis.mapper.GrainAnalysisMonthdataMapper;
import com.chinaitop.depot.grainAnalysis.mapper.GrainAnalysisTestdataMapper;
import com.chinaitop.depot.grainAnalysis.mapper.StorageGrainAnalysisMapper;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysis;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisExample;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisMonthdata;
import com.chinaitop.depot.grainAnalysis.model.GrainAnalysisTestdata;
import com.chinaitop.depot.grainAnalysis.model.StorageGrainAnalysis;
import com.chinaitop.depot.grainAnalysis.model.StorageGrainAnalysisExample;
import com.chinaitop.depot.grainAnalysis.service.GrainAnalysisService;
import com.chinaitop.depot.grainAnalysis.service.StorageGrainAnalysisService;
import com.chinaitop.depot.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;

@Service
public class StorageGrainAnalysisServiceImpl implements StorageGrainAnalysisService {

	@Resource
	private StorageGrainAnalysisMapper storageGrainAnalysisMapper;

	@Resource
	private GrainAnalysisMapper grainAnalysisMapper;
	
	@Resource
	private GrainAnalysisService grainAnalysisService;

	@Resource
	private GrainAnalysisTestdataMapper grainAnalysisTestdataMapper;

	@Resource
	private GrainAnalysisMonthdataMapper grainAnalysisMonthdataMapper;

	@Override
	public List<StorageGrainAnalysis> getList(String startTime, String endTime, Integer orgId, String dataType, String ssyf, String dataStatus) throws Exception {

		StorageGrainAnalysisExample example = new StorageGrainAnalysisExample();
		StorageGrainAnalysisExample.Criteria criteria = example.createCriteria();

		if (StringUtils.isNotBlank(startTime)) {
			criteria.andTjrqGreaterThanOrEqualTo(ParameterUtil.string2datetime(startTime+" 00:00:00"));
		}
		if (StringUtils.isNotBlank(endTime)) {
			criteria.andTjrqLessThanOrEqualTo(ParameterUtil.string2datetime(endTime+" 23:59:59"));
		}
		if (null != orgId) {
			criteria.andOrgIdEqualTo(orgId);
		}
		if (StringUtils.isNotBlank(dataType)) {
			criteria.andDataTypeEqualTo(dataType);
		}
		if (StringUtils.isNotBlank(ssyf)) {
			criteria.andSsyfEqualTo(ssyf);
		}
		if (StringUtils.isNotBlank(dataStatus)) {
			criteria.andDataStatusEqualTo(dataStatus);
		}

		//按照提交日期倒序排序
		example.setOrderByClause("tjrq desc");

		List<StorageGrainAnalysis> list = storageGrainAnalysisMapper.selectByExample(example);

		return list;
	}

	@Override
	public List<Map<String, Object>> selectClcf(String orgId) throws Exception {
		return storageGrainAnalysisMapper.selectClcf(orgId);
	}

	@Override
	public List<GrainAnalysis> selectGrainAnalysis(String id) throws Exception {
		GrainAnalysisExample example = new GrainAnalysisExample();
		GrainAnalysisExample.Criteria criteria = example.createCriteria();
		criteria.andZidEqualTo(id);
		example.setOrderByClause("house_id");
		List<GrainAnalysis> list = grainAnalysisMapper.selectByExample(example);
		
		return list;
	}

	@Override
	public void updateStorageGrainAnalysis(String id) throws Exception {
		// TODO Auto-generated method stub

	}

	@Override
	public void updateGrainAnalysis(String id) throws Exception {
		// TODO Auto-generated method stub

	}

	@Override
	public void removeStorageGrainAnalysis(String id) throws Exception {
		// TODO Auto-generated method stub

	}

	@Override
	public void removeGrainAnalysis(String id) throws Exception {
		// TODO Auto-generated method stub

	}

	@Override
	public Map<String, String> save(String grainJson, String houseGrainJson) throws Exception {
		Map<String, String> map = new HashMap<>();
		
		ObjectMapper mapper = new ObjectMapper();
		StorageGrainAnalysis storageGrainAnalysis = (StorageGrainAnalysis)mapper.readValue(grainJson, StorageGrainAnalysis.class);
		
		JSONArray jsonArray = JSONArray.parseArray(houseGrainJson);
		if (jsonArray.size() > 0) {
			storageGrainAnalysis.setId(UUID.randomUUID().toString().replace("-", ""));
			Date date = new Date();
			storageGrainAnalysis.setCjsj(date);
			storageGrainAnalysis.setUpdatetime(date);
			
			List<GrainAnalysis> grainAnalysis_list = new ArrayList<>();
			List<GrainAnalysisTestdata> testdata_listAll = new ArrayList<>();
			List<GrainAnalysisMonthdata> monthdata_listAll = new ArrayList<>();
			
			for (int i = 0; i < jsonArray.size(); i++) {
				JSONObject jsonObject = (JSONObject) jsonArray.get(i);
				GrainAnalysis grainAnalysis = new GrainAnalysis();
				grainAnalysis.setId(UUID.randomUUID().toString().replace("-", ""));
				grainAnalysis.setOrgId(storageGrainAnalysis.getOrgId());
				grainAnalysis.setHouseId(Integer.parseInt(jsonObject.getString("ch")));
				grainAnalysis.setWarehouseId(Integer.parseInt(jsonObject.getString("hwh")));
				grainAnalysis.setHouseType(Integer.parseInt(jsonObject.getString("cflx")));
				grainAnalysis.setLspz(Integer.parseInt(jsonObject.getString("pz")));
				grainAnalysis.setMxpz(Integer.parseInt(jsonObject.getString("mxpz")));
				grainAnalysis.setZrbgy(jsonObject.getString("bgy"));
				grainAnalysis.setTjr(storageGrainAnalysis.getTjr());
				grainAnalysis.setJcqsrq(storageGrainAnalysis.getJcksrq());
				grainAnalysis.setJczzrq(storageGrainAnalysis.getJcjsrq());
				grainAnalysis.setTjrq(storageGrainAnalysis.getTjrq());
				grainAnalysis.setCreateTime(date);
				grainAnalysis.setUpdatetime(date);
				grainAnalysis.setType(Integer.parseInt(storageGrainAnalysis.getDataType()));
				grainAnalysis.setJcnf(Integer.parseInt(storageGrainAnalysis.getSsyf().substring(0, 4)));
//				grainAnalysis.setJcyf(Integer.parseInt(storageGrainAnalysis.getSsyf().substring(5, 7)));
				grainAnalysis.setZid(storageGrainAnalysis.getId());
				grainAnalysis_list.add(grainAnalysis);

				/* 只有周报才保存粮情数据 */
				if ("1".equals(storageGrainAnalysis.getDataType())) {//只有周报才保存粮情数据
					List<GrainAnalysisTestdata> testdata_list = getTemperatureDataNewList(
							ParameterUtil.datetime2string(storageGrainAnalysis.getJcksrq()), 
							ParameterUtil.date2string1(storageGrainAnalysis.getJcjsrq())+" 23:59:59", 
							grainAnalysis.getHouseId()
							);
					for (GrainAnalysisTestdata t : testdata_list) {
						t.setId(UUID.randomUUID().toString().replace("-", ""));
						t.setGrainAnalysisId(grainAnalysis.getId());
						t.setCjsj(date);
						t.setUpdatetime(date);
						testdata_listAll.add(t);
					}
				}
				/* 只有季报才保存对应季度的月报粮情数据 */
				if ("3".equals(storageGrainAnalysis.getDataType())) {
					Map<String, Object> prama_map = new HashMap<>();
					prama_map.put("houseId", grainAnalysis.getHouseId());
					
					StringBuffer sbf = new StringBuffer(30);
					//年份
					String jcnd = storageGrainAnalysis.getSbzq().substring(0, 4);
					//获取季度
					if (storageGrainAnalysis.getSbzq().indexOf("一") != -1) {//一季度
						sbf.append("('").append(jcnd).append("-01','").append(jcnd).append("-02','").append(jcnd).append("-03')");
					} else if (storageGrainAnalysis.getSbzq().indexOf("二") != -1) {
						sbf.append("('").append(jcnd).append("-04','").append(jcnd).append("-05','").append(jcnd).append("-06')");
					} else if (storageGrainAnalysis.getSbzq().indexOf("三") != -1) {
						sbf.append("('").append(jcnd).append("-07','").append(jcnd).append("-08','").append(jcnd).append("-09')");
					} else if (storageGrainAnalysis.getSbzq().indexOf("四") != -1) {
						sbf.append("('").append(jcnd).append("-10','").append(jcnd).append("-11','").append(jcnd).append("-12')");
					}
					prama_map.put("ssyf", sbf.toString());
					
					List<GrainAnalysisMonthdata> monthdata_list = grainAnalysisMonthdataMapper.queryMonthAnalysis(prama_map);
					for (GrainAnalysisMonthdata m : monthdata_list) {
						m.setId(UUID.randomUUID().toString().replace("-", ""));
						m.setGrainAnalysisId(grainAnalysis.getId());
						m.setCjsj(date);
						m.setUpdatetime(date);
						monthdata_listAll.add(m);
					}
				}
			}
			//保存单仓粮情主表数据
			for (GrainAnalysis g : grainAnalysis_list) {
				grainAnalysisMapper.insert(g);
			}
			//保存单仓粮情对应的温度表数据
			for (GrainAnalysisTestdata t : testdata_listAll) {
				grainAnalysisTestdataMapper.insert(t);
			}
			//保存季报下单仓粮情对应的月报表数据
			for (GrainAnalysisMonthdata t : monthdata_listAll) {
				grainAnalysisMonthdataMapper.insert(t);
			}

			//保存报表主表数据
			storageGrainAnalysisMapper.insert(storageGrainAnalysis);

			map.put("status", "200");
			map.put("msg", storageGrainAnalysis.getId());
		}

		return map;
	}

	@Override
	public StorageGrainAnalysis findByIdStorageGrainAnalysis(String id) throws Exception {
		StorageGrainAnalysis storageGrainAnalysis = storageGrainAnalysisMapper.selectByPrimaryKey(id);
		return storageGrainAnalysis;
	}

	@Override
	public List<GrainAnalysisTestdata> getTemperatureDataNewList(String jcqsrq, String jczzrq, Integer houseId)
			throws Exception {
		Map<String, Object> param_map = new HashMap<>();
		param_map.put("houseId", houseId);
		param_map.put("starttime", ParameterUtil.string2datetime(jcqsrq+" 00:00:00"));
		param_map.put("endtime", ParameterUtil.string2datetime(jczzrq+" 23:59:59"));
		List<GrainAnalysisTestdata> testdata_list = grainAnalysisTestdataMapper.selectHouseTestdata(param_map);
		return testdata_list;
	}

	@Override
	public Map<String, String> submitStorageGrainAnalysis(String id) throws Exception {
		GrainAnalysisExample grainAnalysisExample = new GrainAnalysisExample();
		GrainAnalysisExample.Criteria grainAnalysisCriteria = grainAnalysisExample.createCriteria();
		grainAnalysisCriteria.andZidEqualTo(id);
		List<GrainAnalysis> list = grainAnalysisMapper.selectByExample(grainAnalysisExample);
		
		StorageGrainAnalysis storageGrainAnalysis = storageGrainAnalysisMapper.selectByPrimaryKey(id);
		storageGrainAnalysis.setDataStatus("1");
		storageGrainAnalysis.setSbcfs(list.size());
		storageGrainAnalysis.setUpdatetime(new Date());
		storageGrainAnalysisMapper.updateByPrimaryKey(storageGrainAnalysis);
		Map<String, String> map = new HashMap<>();
		map.put("status", "200");
		map.put("msg", "操作成功");
		return map;
	}

	@Override
	public List<StorageGrainAnalysis> getListSpt(String startTime, String endTime, String kqbm, String dataType,
			String ssyf) throws Exception {
		Map<String, Object> map = new HashMap<>();
		if (StringUtils.isNotBlank(startTime)) {
			map.put("startTime", ParameterUtil.string2datetime(startTime));
		}
		if (StringUtils.isNotBlank(endTime)) {
			map.put("endTime", ParameterUtil.string2datetime(endTime));
		}
		if (StringUtils.isNotBlank(kqbm)) {
			List<String> l = new ArrayList<>();
			String []str_num = kqbm.split(",");
			for (String s : str_num) {
				l.add(s);
			}
			map.put("kqbmList", l);
		}
		if (StringUtils.isNotBlank(dataType)) {
			map.put("dataType", dataType);
		}
		if (StringUtils.isNotBlank(ssyf)) {
			map.put("ssyf", "%"+ssyf+"%");
		}
		List<StorageGrainAnalysis> list = storageGrainAnalysisMapper.selectByExampleSpt(map);
		return list;
	}

	@Override
	public List<GrainAnalysis> selectGrainAnalysisSpt(String zid) throws Exception {
		List<GrainAnalysis> list = grainAnalysisMapper.selectByExampleSpt(zid);
		return list;
	}


}
