package com.chinaitop.depot.keeperAccount.controller;

import java.util.List;

import javax.annotation.Resource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.MediaType;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.chinaitop.depot.keeperAccount.model.StorageFcbgz;
import com.chinaitop.depot.keeperAccount.service.StorageFcbgzService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping(value="/storageFcbgzController")
@Api(value= "StorageFcbgzController", description = "新版保管账控制类")
public class StorageFcbgzController {

	private static final Logger logger = LoggerFactory.getLogger(StorageFcbgzController.class);

	@Resource
	private StorageFcbgzService storageFcbgzService;

	
	@RequestMapping(value="/getListFcbgz", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.GET)
	@ApiOperation(value="新版分仓保管账列表", notes = "支持分页")
    @ApiImplicitParams({
    	@ApiImplicitParam(name="pageNum", value="页数", paramType="query"),
    	@ApiImplicitParam(name="pageSize", value="每页显示条数", paramType="query"),
    	@ApiImplicitParam(name="lspz", value="粮食品种", paramType="query"),
    	@ApiImplicitParam(name="lsxz", value="粮食性质", paramType="query"),
    	@ApiImplicitParam(name="jznd", value="记账年度", paramType="query"),
    	@ApiImplicitParam(name="dataStatus", value="是否记账(0:已记账,1:未记账)", paramType="query"),
    	@ApiImplicitParam(name="sfjz", value="是否结转(0:是,1:否)", paramType="query"),
    	@ApiImplicitParam(name="orgId", value="库ID", paramType="query")
    })
	public PageInfo<StorageFcbgz> getListFcbgz(Integer pageNum, Integer pageSize, 
			Integer lspz, Integer lsxz, String jznd, String dataStatus, String sfjz, Integer orgId) {

		List<StorageFcbgz> list = null;

		if (pageNum!=null && pageSize!=null) {
			PageHelper.startPage(pageNum, pageSize);
		}
		try {
			list = storageFcbgzService.selectFcbgzList(lspz, lsxz, jznd, dataStatus, sfjz, orgId);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		
		PageInfo<StorageFcbgz> pageInfo = new PageInfo<StorageFcbgz>(list);
		return pageInfo;
	}
	
	@RequestMapping(value="/queryJzList", method = RequestMethod.GET)
	@ApiOperation(value="点记账", notes = "")
	@ApiImplicitParams({
    	@ApiImplicitParam(name="ch", value="仓房ID", paramType="query"),
    	@ApiImplicitParam(name="hwh", value="货位ID", paramType="query"),
    	@ApiImplicitParam(name="lsxz", value="粮食性质ID", paramType="query"),
    	@ApiImplicitParam(name="jznd", value="记账年度", paramType="query")
    })
	public List<StorageFcbgz> queryJzList(Integer ch, Integer hwh, Integer lsxz, String jznd) {
		List<StorageFcbgz> list = null;
		try {
			list = storageFcbgzService.queryJzList(ch, hwh, lsxz, jznd);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return list;
	}
	
	@Scheduled(cron = "00 00 23 * * ?")
	@RequestMapping(value="/timedGenerationFcbgzData", method = RequestMethod.GET)
	@ApiOperation(value="定时生成保管账数据", notes = "")
	@ApiImplicitParams({
    	@ApiImplicitParam(name="orgId", value="库ID", paramType="query"),
    	@ApiImplicitParam(name="hwxz", value="粮食性质", paramType="query"),
    	@ApiImplicitParam(name="rqs", value="记账日期", paramType="query")
    })
	public void timedGenerationFcbgzData() {
		try {
			storageFcbgzService.timedGenerationFcbgzData(null, null, null);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		System.out.println("结束了");
	}
	
	
	
	@RequestMapping(value="/timedGenerationFcbgzData_test", method = RequestMethod.GET)
	@ApiOperation(value="定时生成保管账数据_测试接口", notes = "")
	@ApiImplicitParams({
    	@ApiImplicitParam(name="orgId", value="库ID", paramType="query"),
    	@ApiImplicitParam(name="hwxz", value="粮食性质", paramType="query"),
    	@ApiImplicitParam(name="rqs", value="记账日期", paramType="query")
    })
	public void timedGenerationFcbgzData_test(String orgId, String hwxz, String rqs) {
		try {
			storageFcbgzService.timedGenerationFcbgzData(orgId, hwxz, rqs);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		System.out.println("结束了");
	}
}
