package com.chinaitop.depot.keeperAccount.service.impl;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.UUID;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import com.chinaitop.depot.keeperAccount.mapper.StorageFcbgzMapper;
import com.chinaitop.depot.keeperAccount.model.StorageFcbgz;
import com.chinaitop.depot.keeperAccount.model.StorageFcbgzExample;
import com.chinaitop.depot.utils.ParameterUtil;

@Service
public class FcbgzUtilsServiceImpl {

	@Resource
	private StorageFcbgzMapper storageFcbgzMapper;

	/**
	 * 查询一个货位在某年有没有结转
	 * @param hwh
	 * @param jznd
	 * @return
	 * @throws Exception
	 */
	public boolean topYearJzData(Integer hwh, Integer jznd) throws Exception {
		boolean flag = false;
		StorageFcbgzExample example = new StorageFcbgzExample();
		StorageFcbgzExample.Criteria criteria = example.createCriteria();
		criteria.andHwhEqualTo(hwh);
		criteria.andJzndEqualTo(jznd.toString());
		List<StorageFcbgz> list = storageFcbgzMapper.selectByExample(example);
		if (null != list && list.size() > 0) {
			flag = true;
		}
		return flag;
	}

	/**
	 * 新增或修改一条合计账
	 * @param f
	 */
	public void saveOrUpdateHjz(StorageFcbgz f)  throws Exception {
		StorageFcbgzExample example = new StorageFcbgzExample();
		StorageFcbgzExample.Criteria criteria = example.createCriteria();
		criteria.andHwhEqualTo(f.getHwh());//货位
		criteria.andDataTypeEqualTo("1");//合计账
		criteria.andYwlxEqualTo(f.getYwlx());//业务类型
		criteria.andSjfmEqualTo(ParameterUtil.date2string1(f.getJzrq()).substring(0, 7));//年月
		List<StorageFcbgz> list = storageFcbgzMapper.selectByExample(example);
		StorageFcbgz sf = null;
		if (null != list && list.size() > 0) {
			sf = list.get(0);

			//收入数量
			BigDecimal srsl = f.getSrsl().add(sf.getSrsl()).setScale(3, BigDecimal.ROUND_DOWN);
			sf.setSrsl(srsl);

			//支出数量
			BigDecimal zcsl = f.getZcsl().add(sf.getZcsl()).setScale(3, BigDecimal.ROUND_DOWN);
			sf.setZcsl(zcsl);

			//损耗数量
			BigDecimal shsl = f.getShsl().add(sf.getShsl()).setScale(3, BigDecimal.ROUND_DOWN);
			sf.setShsl(shsl);
			
			//溢余数量
			BigDecimal yysl = f.getYysl().add(sf.getYysl()).setScale(3, BigDecimal.ROUND_DOWN);
			sf.setYysl(yysl);
			
			//库存数量
			sf.setKcsl(f.getKcsl());
			
			storageFcbgzMapper.updateByPrimaryKey(sf);
		} else {
			sf = f;
			sf.setId(UUID.randomUUID().toString().replace("-", ""));
			sf.setSjfm(ParameterUtil.date2string1(f.getJzrq()).substring(0, 7));
			sf.setDataType("1");
			sf.setDataStatus("0");
			sf.setCjsj(new Date());
			storageFcbgzMapper.insert(sf);
		}
	}

	/**
	 * 查询集合
	 * @param order 排序方式
	 * @param hwh
	 * @param lspz
	 * @param lsxz
	 * @param jznd
	 * @param sjfm 数据说明
	 * @param dataType
	 * @return
	 * @throws Exception
	 */
	public List<StorageFcbgz> getList(String order, Integer ch, Integer hwh, Integer lspz, Integer lsxz, String jznd, String sjfm, String dataType)  throws Exception {

		StorageFcbgzExample example = new StorageFcbgzExample();
		StorageFcbgzExample.Criteria criteria = example.createCriteria();
		if (StringUtils.isNotBlank(order)) {
			example.setOrderByClause(order);//排序方式
		}
		if (null != ch) {
			criteria.andChEqualTo(ch);
		}
		if (null != hwh) {
			criteria.andHwhEqualTo(hwh);
		}
		if (null != lspz) {
			criteria.andMxpzEqualTo(lspz);
		}
		if (null != lsxz) {
			criteria.andLyxzEqualTo(lsxz);
		}
		if (StringUtils.isNotBlank(jznd)) {
			criteria.andJzndEqualTo(jznd);
		}
		if (StringUtils.isNotBlank(sjfm)) {
			criteria.andSjfmEqualTo(sjfm);
		}
		if (StringUtils.isNotBlank(dataType)) {
			criteria.andDataTypeEqualTo(dataType);
		}
		List<StorageFcbgz> list = storageFcbgzMapper.selectByExample(example);

		return list;
	}
	
	/**
	 * 初始化1-12月的数据
	 * 
	 * @param zzdm 18位统一性用社代码
	 * @param orgId 单位ID
	 * @return
	 */
	public String[] getDayNum() {
		String []day_num = new String[12];
		String day = "";
		for (int i = 0; i < 12; i++) {
			day = String.format("%02d", i+1);
			day_num[i] = day;
			System.out.println(day_num[i]);
		}
		return day_num;
	}
	
	public static void main(String[] args) {
		FcbgzUtilsServiceImpl f = new FcbgzUtilsServiceImpl();
		f.getDayNum();
	}
}
