package com.chinaitop.depot.keeperAccount.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import com.chinaitop.depot.keeperAccount.mapper.StorageFcbgzMapper;
import com.chinaitop.depot.keeperAccount.model.StorageFcbgz;
import com.chinaitop.depot.keeperAccount.service.StorageFcbgzService;
import com.chinaitop.depot.utils.ParameterUtil;

@Service
public class StorageFcbgzServiceImpl implements StorageFcbgzService {

	@Resource
	private StorageFcbgzMapper storageFcbgzMapper;

	@Resource
	private FcbgzUtilsServiceImpl fcbgzUtilsServiceImpl;

	@Override
	public void timedGenerationFcbgzData(String orgId, String hwxz, String rqs) throws Exception {
		
		Map<String, Object> map = new HashMap<>();
		if (StringUtils.isNotBlank(orgId)) {
			map.put("orgId", orgId);
		}
		if (StringUtils.isNotBlank(hwxz)) {
			map.put("hwxz", "("+hwxz+")");
		} else {
			map.put("lsxz", "(6877,6878,6879,3052)");//查省级、市级、县级储备粮和商品粮
		}
		if (StringUtils.isNotBlank(rqs)) {
			map.put("rqs", rqs);
		} else {
			map.put("rqs", ParameterUtil.getSysDate());
		}

		List<StorageFcbgz> list = storageFcbgzMapper.queryCrkFcbgzGroupByData(map);
		
		if (null != list && list.size() > 0) {
			StorageFcbgz f = null;
			for (int i = 0; i < list.size(); i++) {
				f = list.get(i);

				//先查询这个货位在上一年有没有结转，不结转不能继续生成数据
				boolean flag = fcbgzUtilsServiceImpl.topYearJzData(f.getHwh(), Integer.parseInt(f.getJznd())-1);

				f.setId(UUID.randomUUID().toString().replace("-", ""));
				f.setCjsj(new Date());
				f.setSjfm(ParameterUtil.date2string1(f.getJzrq()).substring(0, 7));
				if (flag) {
					f.setIsShow("0");
				} else {
					f.setIsShow("1");
				}

				storageFcbgzMapper.insert(f);
				
				//生成或修改合计数据
				fcbgzUtilsServiceImpl.saveOrUpdateHjz(f);
			}
		}
	}

	@Override
	public List<StorageFcbgz> selectFcbgzList(Integer lspz, Integer lsxz, String jznd, String dataStatus,
			String sfjz, Integer orgId) {
		
		Map<String, Object> map = new HashMap<>();
		if (lspz != null) {
			map.put("mxpz", lspz);
		}
		if (lsxz != null) {
			map.put("lyxz", lsxz);
		}
		if (StringUtils.isNotBlank(jznd)) {
			map.put("jznd", jznd);
		} else {
			String nf = ParameterUtil.getSysDate().substring(0, 4);
			map.put("jznd", nf);
		}
		if (StringUtils.isNotBlank(dataStatus)) {
			map.put("dataStatus", dataStatus);
		}
		if (StringUtils.isNotBlank(sfjz)) {
			map.put("sfjz", sfjz);
		}
		if (null != orgId) {
			map.put("orgId", orgId);
		}
		List<StorageFcbgz> list = storageFcbgzMapper.selectFcbgzList(map);
		return list;
	}

	@Override
	public List<StorageFcbgz> queryJzList(Integer ch, Integer hwh, Integer lsxz, String jznd) throws Exception {
		/**
		 * 实现思路：数据由上年结转数据、[jznd]年的每月数据、[jznd]年需要结转的总数数据组成
		 */
		//用来存放返回的所有数据
		List<StorageFcbgz> list = new ArrayList<>();
		
		/** 先查上一年的结转数据  start  **/
		Integer top_year = Integer.parseInt(jznd)-1;
		List<StorageFcbgz> top_year_jz_list = fcbgzUtilsServiceImpl.getList(null, ch, hwh, null, null, top_year.toString(), null, "2");
		StorageFcbgz top_year_jz = null;
		if (null != top_year_jz_list && top_year_jz_list.size() > 0) {
			top_year_jz = top_year_jz_list.get(0);
		} else {
			top_year_jz = new StorageFcbgz();
			top_year_jz.setDataType("2");
			top_year_jz.setKcsl(new BigDecimal(0));
		}
		list.add(top_year_jz);
		
		/** 先查上一年的结转数据  end  **/

		
		/** 查[jznd]年的每月数据  start  **/
		List<StorageFcbgz> jznd_month_jz_list = null; //每月的明细
		List<StorageFcbgz> jznd_month_jz_sum_list = null; //每月的合计
		StorageFcbgz year_jz_obj = null;
		for (int i = 0; i < 12; i++) {
			//从1-12月依次查， 先查每月的明细
			String jzrq = jznd+"-"+String.format("%02d", i+1).trim();
			jznd_month_jz_list = fcbgzUtilsServiceImpl.getList("last_business_date", ch, hwh, null, lsxz, jznd, jzrq, "0");
			if (null != jznd_month_jz_list && jznd_month_jz_list.size() > 0) {
				for (StorageFcbgz f_day : jznd_month_jz_list) {
					list.add(f_day);
					
					//预设[jznd]年的年合计数据就是这年当前最后一条生成的数据
					year_jz_obj = f_day;
					year_jz_obj.setDataType("2");//年合计
					year_jz_obj.setDataStatus("1");//未记账
				}
				//查询这个月的合计数据
				jznd_month_jz_sum_list = fcbgzUtilsServiceImpl.getList(null, ch, hwh, null, null, jznd, null, "1");
				if (null != jznd_month_jz_sum_list && jznd_month_jz_sum_list.size() > 0) {
					list.add(jznd_month_jz_sum_list.get(0));
				}
			} else {
				//要是没有这个月的数据，那就跳过这个月
				continue;
			}
		}
		/** 查[jznd]年的每月数据  end  **/


		/** 查[jznd]年的年合计数据  start  **/
		List<StorageFcbgz> jznd_year_jz_sum_list = fcbgzUtilsServiceImpl.getList(null, ch, hwh, null, null, jznd, null, "2"); //[jznd]年的合计
		if (null != jznd_year_jz_sum_list && jznd_year_jz_sum_list.size() > 0) {
			year_jz_obj = jznd_year_jz_sum_list.get(0);
		}
		if (null != year_jz_obj) {
			list.add(year_jz_obj);
		}
		/** 查[jznd]年的年合计数据  start  **/
		
		return list;
	}

	
}
